module CT = Cduce_lib.Types

let trace msg =
  output_string stderr (msg ^ "\n"); flush stderr;;

let debug_string msg =
  trace ("CDuce debug:" ^ msg);;

let print_arg_noname arg =
  let b = Buffer.create 1024 in 
  let ppf = Format.formatter_of_buffer b in
  CT.Print.print_noname ppf arg;
  Format.pp_print_flush ppf ();
  Buffer.contents b;;

let print_arg (i,j) =
  output_string stderr ("argument = (" ^
    (CT.Print.to_string i) ^ " {" ^ (print_arg_noname i)
  ^ "} -> " ^  
    (CT.Print.to_string
      j) ^ ")\n");
  output_string stderr ("-----------------\n");;

let rec convert_service = function
  | CT.TInt n -> "<param type=\"int\">\"" ^ n ^ "\""
  | CT.TString n -> "<param type=\"string\">\"" ^ n ^ "\""
  | CT.TFile n -> "<param type=\"file\">\"" ^ n ^ "\""
  | CT.TProd (e1,e2) -> (convert_service e1) ^ " " ^ 
      (convert_service e2)
  | CT.TOption e -> "<opt>[ " ^ (convert_service e) ^ " ]"
  | CT.TSuffix(_,e) -> "<suffix>[ " ^ (convert_service e) ^ " ]"
  | CT.TConst n -> "<const_suffix>\"" ^ n ^ "\""
  | CT.TESuffix n -> "<end_suffix>\"" ^ n ^ "\""
  | CT.TSum (e1,e2) -> "<sum>[<inj1>[" ^ (convert_service e1) ^
      "] <inj2>[" ^ (convert_service e2) ^ "]]"
  | CT.TList (name,e) -> "<list name=\"" ^ name ^ "\">[" ^
      (convert_service e) ^ "]"
  | CT.TAny -> "<any>[]";
  | _ -> assert false;;

let convert_arg (arg,ret) =
  print_arg (arg,ret);
  let service = CT.Service.to_service_params arg in
  trace (CT.Service.to_string service); 
  ("[ " ^ (convert_service service) ^ " ];;");;

let convert_fun_param p =
  match p with 
    | Cduce_lib.Value.Abstraction (args,f) ->
	(match args with
	  | None -> trace "no parameter"; ""
	  | Some la ->  convert_arg (List.hd la))
    | _ -> "" ;;
